////////////////////////////////////////////////////////////////////// 
// regress/RegressAnalysis.cpp 
// (c) 2000-2001 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Friday June 21, 2002
// 
 
#include "RegressAnalysis.h"

// Trait model parameters
//

double RegressionAnalysis::mean = 0.0;
double RegressionAnalysis::variance = 1.0;
double RegressionAnalysis::heritability = 0.5;
bool RegressionAnalysis::rankFamilies = false;

// Constructor and destructor
//

RegressionAnalysis::RegressionAnalysis(Pedigree & pedigree)
   : MerlinCore(pedigree), kinship(mantra)
   {
   regress = NULL;
   }

RegressionAnalysis::~RegressionAnalysis()
   {
   if (regress != NULL)
      delete [] regress;
   }

// Setup global storage
//

void RegressionAnalysis::SetupGlobals()
   {
   MerlinCore::SetupGlobals();

   if (regress != NULL)
      delete [] regress;

   regress = new FancyRegression[ped.traitCount];

   for (int i = 0; i < ped.traitCount; i++)
      {
      regress[i].trait = i;
      regress[i].mean = mean;
      regress[i].variance = variance;
      regress[i].heritability = heritability;
      }

   if (rankFamilies)
      RankFamilies();

   betweenMarkers = scanChromosome = multipoint = true;
   }

void RegressionAnalysis::RankFamilies()
   {
   printf("Family Informativeness\n"
       "======================\n"
       "%15s %15s %7s %7s %7s %7s %7s\n",
       "Family", "Trait", "People", "Phenos", "Pairs", "Info", "ELOD20");

   IntArray totalCount(ped.traitCount); totalCount.Zero();
   IntArray totalPheno(ped.traitCount); totalPheno.Zero();
   IntArray totalPairs(ped.traitCount); totalPairs.Zero();

   for (int trait = 0; trait < ped.traitCount; trait++)
      regress[trait].ResetMaxInfo();

   for (int i = 0; i < ped.familyCount; i++)
      {
      Family * family = ped.families[i];

      mantra.Prepare(ped, *family);

      if (mantra.bit_count > maxBits)
         {
         printf("%15s *** %d-bit family skipped ***\n",
            (const char *) family->famid, mantra.bit_count);
         continue;
         }

      mantra.PrepareIBD();
      tree.MakeMinimalTree(1.0, mantra.bit_count);
      kinship.SelectFamily(&ped, family);
      kinship.Calculate(tree);

      for (int trait = 0; trait < ped.traitCount; trait++)
         {
         regress[trait].SetupFamily(kinship);

         int phenotypes = 0;
         for (int j = family->first; j <= family->last; j++)
            if (ped[j].isPhenotyped(trait))
               phenotypes++;

         totalCount[trait] += family->count;
         totalPheno[trait] += phenotypes;
         totalPairs[trait] += regress[trait].CountPairs();

         printf("%15s %15s %7d %7d %7d %7.3f %7.3f\n",
            (const char *) family->famid,
            (const char *) ped.traitNames[trait],
            family->count, phenotypes, regress[trait].CountPairs(),
            regress[trait].maxInfo,
            regress[trait].maxInfo * 0.04 * 0.2171472409516);
         }
      }

   for (int trait = 0; trait < ped.traitCount; trait++)
      printf("%15s %15s %7d %7d %7d %7.3f %7.3f\n",
         "TOTAL", (const char *) ped.traitNames[trait],
         totalCount[trait], totalPheno[trait], totalPairs[trait],
         regress[trait].sumMaxInfo,
         regress[trait].sumMaxInfo * 0.04 * 0.2171472409516);

   printf("\n");
   }

// Setup storage that depends on map-length
//

int RegressionAnalysis::SetupMap(int chromosome)
   {
   chromosome = MerlinCore::SetupMap(chromosome);

   for (int i = 0; i < ped.traitCount; i++)
      {
      regress[i].SetPositionCount(analysisPositions.Length());
      regress[i].ResetMaxInfo();
      }

   return chromosome;
   }

// Setup analysis for each family
//

bool RegressionAnalysis::SelectFamily(Family * f)
   {
   if (MerlinCore::SelectFamily(f))
      {
      kinship.SelectFamily(&ped, f);

      ProgressReport("Preparing Matrices", 0, ped.traitCount + 1);
      tree.MakeMinimalTree(1.0, mantra.bit_count);
      kinship.Calculate(tree);

      for (int i = 0; i < ped.traitCount; i++)
         {
         ProgressReport("Preparing Matrices", i+1, ped.traitCount + 1);
         regress[i].SetupFamily(kinship);
         }

      return true;
      }

   return false;
   }

// Carry out analysis for a family
//

void RegressionAnalysis::AnalyseLocation(int pos, Multipoint & inheritance)
   {
   kinship.Calculate(inheritance);

   for (int i = 0; i < ped.traitCount; i++)
      regress[i].Analyse(kinship, pos);
   }

// Put it all together
//

void RegressionAnalysis::Analyse()
   {
   singlepoint = NULL;
   right = NULL;

   try
      {
      BasicTree::FreeSwap();

      ScoreSinglepoint();

      if (informativeCount != 0)
         {
         if (ScoreConditionals())
            ScanChromosome();
         else
            PrintMessage("  SKIPPED: Requires impossible recombination pattern");

         if (!twopoint) delete [] right;
         }

      CleanMessages();
      delete [] singlepoint;
      }
   catch (const TreesTooBig & problem)
      {
      PrintMessage("  SKIPPED: At least %d megabytes needed", problem.memory_request);
      CleanMessages();

      if (singlepoint != NULL) delete [] singlepoint;
      if (right != NULL) delete [] right;
      }
   catch (const OutOfTime & problem)
      {
      PrintMessage("  SKIPPED: Analysis would require more than %d minutes\n", maxMinutes);
      CleanMessages();

      if (singlepoint != NULL) delete [] singlepoint;
      if (right != NULL) delete [] right;
      };
   };

// Output LOD scores at each location
//

void RegressionAnalysis::PrintScores()
   {
   for (int i = 0; i < ped.traitCount; i++)
      regress[i].PrintScores(ped.traitNames[i], labels);
   }

 
